/*
	$Id$
*/

#include <stdlib.h>
#include <SDL/SDL.h>

#include "taquin.h"
#include "input.h"
#include "debug.h"

/* private */
static void tooglefullscreen(struct Taquin *);

int Input(struct Taquin *taquin, int *x, int *y)
{
	int ret = 0;
	int xnew = *x;
	int ynew = *y;

	if(taquin->demo || SDL_PollEvent(&taquin->event))
	{
		DEBUG("event\n");

		ret = 1;

		/* some initial random key events */
		if(taquin->demo)
		{
			int rnd = rand();

			taquin->demo -= 1;
			taquin->event.type = SDL_KEYDOWN;

			if((rnd/4)*4 == rnd)
			{
				rnd = 3;
			}
			else if((rnd/3)*3 == rnd)
			{
				rnd = 2;
			}
			else if((rnd/2)*2 == rnd)
			{
				rnd = 1;
			}
			else
			{
				rnd = 0;
			}
			taquin->event.key.keysym.sym = SDLK_UP + rnd;
			DEBUG("rnd : %d\n", taquin->event.key.keysym.sym - SDLK_UP);
		}

		if(!taquin->quit)
		{
			switch(taquin->event.type)
			{
				case SDL_KEYDOWN:
					DEBUG("key down\n");
					switch(taquin->event.key.keysym.sym)
					{
						case SDLK_LEFT:
							if(xnew < 2)
							{
								xnew += 1;
							}
							break;

						case SDLK_RIGHT:
							if(xnew > 0)
							{
								xnew -= 1;
							}
							break;

						case SDLK_UP:
							if(ynew < 2)
							{
								ynew += 1;
							}
							break;

						case SDLK_DOWN:
							if(ynew > 0)
							{
								ynew -= 1;
							}
							break;

						case SDLK_TAB:
							tooglefullscreen(taquin);
							break;

						case SDLK_ESCAPE:
							taquin->quit = 1;
							break;

						default:
							break;
					}
					break;

				case SDL_KEYUP:
					break;

				case SDL_VIDEOEXPOSE:
					taquin->refresh = 1;
					break;

				case SDL_QUIT:
					taquin->quit = 1;
					break;

				default:
					DEBUG("unknown event\n");
					break;
			}
		}

		*x = xnew;
		*y = ynew;
	}

	return ret;
}

static void tooglefullscreen(struct Taquin *taquin)
{
	DEBUG("tooglefullscreen\n");

	if(SDL_WM_ToggleFullScreen(taquin->surface))
	{
		taquin->fullscreen = 1 - taquin->fullscreen;
		taquin->refresh = 1;
		SDL_ShowCursor(taquin->fullscreen ? SDL_DISABLE : SDL_ENABLE);
	}
}


