/*
	$Id$
*/

#include <stdlib.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#include "taquin.h"
#include "input.h"
#include "draw.h"
#include "debug.h"

/* in Hz */
#define FREQUENCY 60

/* private */
static void loop(struct Taquin *taquin);
static void check_end(struct Taquin *taquin, int x, int y);

int Taquin_main(void)
{
	if(0 == SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO))
	{
		struct Taquin *MyTaquin;

		MyTaquin = malloc(sizeof(*MyTaquin));
		if(MyTaquin)
		{
			MyTaquin->title      = "Taquin";
			MyTaquin->fullscreen = 0;
			MyTaquin->refresh    = 0;
			MyTaquin->demo       = 100;
			MyTaquin->quit       = 0;

			MyTaquin->surface = SDL_SetVideoMode(WIDTH, HEIGHT, DEPTH, SDL_SWSURFACE | (MyTaquin->fullscreen ? SDL_FULLSCREEN : 0));
			if(MyTaquin->surface)
			{
				SDL_Surface *MyImage;

				SDL_WM_SetCaption(MyTaquin->title, MyTaquin->title);

				/*
					- load the image
					- clone it in two surfaces using the display pixfmt
					- free the image
					- darken the background surface
					- draw all cases on the other surface
				*/
				MyImage = IMG_Load("taquin.jpg");
				if(MyImage)
				{
					const SDL_VideoInfo *MyInfo = SDL_GetVideoInfo();

					MyTaquin->image = SDL_ConvertSurface(MyImage, MyInfo->vfmt, SDL_SWSURFACE);
					MyTaquin->back  = SDL_ConvertSurface(MyImage, MyInfo->vfmt, SDL_SWSURFACE);
					SDL_FreeSurface(MyImage);
					if(MyTaquin->image && MyTaquin->back)
					{
						Draw_grid(MyTaquin->image, MyInfo->vfmt);
						SDL_SetAlpha(MyTaquin->back, SDL_SRCALPHA, 255*0.50);

						loop(MyTaquin);
					}

				    SDL_FreeSurface(MyTaquin->image);
				    SDL_FreeSurface(MyTaquin->back);
				}

				/*  SDL_Quit() will free the main surface. Yes, SDL sucks. */
			}

			free(MyTaquin);
		}
	}

	return 0;
}

static void loop(struct Taquin *taquin)
{
	int x, y, xnew = 2, ynew = 2; /* the hole is located in the bottom right corner */
	int i;

	DEBUG("loop\n");

	/* init grid */
	for(i=0; i<9; i++)
	{
		taquin->grid[i] = i;
	}

	x = xnew;
	y = ynew;
	Draw_game(taquin, xnew, ynew);

	SDL_Delay(1000*2);

	while(!taquin->quit)
	{
		while(Input(taquin, &xnew, &ynew))
		{
			if((xnew != x) || (ynew != y))
			{
				taquin->grid[x+y*3] = taquin->grid[xnew+ynew*3];
				Draw_game(taquin, xnew, ynew);
				x = xnew;
				y = ynew;

				if(taquin->demo)
				{
					//SDL_Delay(1000/FREQUENCY*5);
				}
				else
				{
					check_end(taquin, xnew, ynew);
				}
			}
			else if(taquin->refresh)
			{
				taquin->refresh = 0;
				SDL_Flip(taquin->surface);
			}
		}

		SDL_Delay(1000/FREQUENCY); /* Required to avoid a busyloop. Yes, SDL blows. */
	}
}

static void check_end(struct Taquin *taquin, int x, int y)
{
	int i, count = 0;

	for(i=0; i<9; i++)
	{
		if(i != (x+y*3))
		{
			if(i == taquin->grid[i])
			{
				count++;
			}
		}
	}

	if(count == 8)
	{
		taquin->quit = 1;

		printf("Victory !\n");
	}
}

